#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File: models/base.py
# Author: MingshiCai i@unoiou.com
# Created Date: 2019-08-06 10:32:15
# ----
# Last Modified:
# Modified By:
# ----
# Copyright (c) 2019 MingshiCai i@unoiou.com
from peewee import SqliteDatabase, Model

db = SqliteDatabase('weibo.db')


class BaseModel(Model):
    class Meta:
        database = db



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File: models/attachment.py
# Author: MingshiCai i@unoiou.com
# Created Date: 2019-08-06 10:32:42
# ----
# Last Modified:
# Modified By:
# ----
# Copyright (c) 2019 MingshiCai i@unoiou.com
from peewee import CharField, ForeignKeyField

from harbor.models.base import BaseModel
from harbor.models.post import Post


class Attachment(BaseModel):
    uri = CharField(max_length=500, null=True)
    post = ForeignKeyField(Post, backref='attachments')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File: spider/main.py
# Author: MingshiCai i@unoiou.com
# Created Date: 2019-08-06 10:36:40
# ----
# Last Modified:
# Modified By:
# ----
# Copyright (c) 2019 MingshiCai i@unoiou.com
import logging
import json
import re

from arrow import get as ar
from bs4 import BeautifulSoup as BS
from requests import get
from tqdm import tqdm
from time import sleep

from harbor.constant import (
    URL_TPL, HEADER_PATH_TPL, REFERER_URL_TPL, LIKE_NUM_PATTERN,
    COMMENT_NUM_PATTERN, REPO_NUM_PATTERN, REPO_TEST_PATTERN, TIME_PATTERN,
    DATE_FMTS, SOURCE_DEVICE_PATTERN, VISIBILITY_PATTERN
)
from harbor.marshallers.factory import Marshaller
from harbor.models.attachment import Attachment
from harbor.models.post import Post

LOGGER = logging.getLogger(__name__)


class Spider:
    """A spider that does everything.

    Not an abstract factory.
    """

    def __init__(self, headers, uid, marshaller_name):
        if not headers['Cookie']:
            raise KeyError("Empty cookie.")
        self._all_page_num = None
        self._all_post_num = 0
        self._current_year = ar().year
        self._current_page = 0
        self._current_html = None
        self._post_item = None
        self._attachment_item = None
        self._uid = uid
        self._headers = headers
        self._marshaller = Marshaller(marshaller_name)

    @property
    def url(self):
        """Dynamically get url and `_current_page` auto increment.
        """
        self._current_page += 1
        return URL_TPL.format(self._uid, self._current_page)

    @property
    def headers(self):
        """Dynamically get `:path` and `referer` header.
        """
        self._headers['path'] = HEADER_PATH_TPL.format(
            self._uid, self._current_page)
        self._headers['referer'] = REFERER_URL_TPL.format(
            self._uid, self._current_page - 1
        )
        return self._headers

    @property
    def has_post(self):
        return True if self._current_page < self._all_page_num else False

    def _set_all_page_num(self):
        """Initialize all page num.
        """
        res = get(self.url, headers=self.headers)
        post_num = re.findall(r'微博\[(\d+)\]', res.text)[0]
        page_num = re.findall(r'\/(\d+)页', res.text)[0]
        self._current_page -= 1
        self._all_page_num = int(page_num)
        self._all_post_num = int(post_num)

    def _get_html(self):
        url = self.url
        res = None
        while True:
            res = get(url, headers=self.headers)
            if res.status_code == 200:
                break
            sleep(4)
        self._current_html = res.text

    def _parse(self):
        """Extract info from HTML content.
        TODO: refractor
        """
        soup = BS(self._current_html, 'lxml')
        for item in soup.select('div.c'):
            temp = {}
            # main content
            ctt = item.select('span.ctt')
            if not ctt:
                continue
            weibo_body = item.select('div')
            if len(weibo_body) > 1:
                temp['content'] = weibo_body[0].text
                btn_group = weibo_body[1].text
            else:
                temp['content'] = weibo_body[0].select('span.ctt')[0].text
                btn_group = weibo_body[0].text
            temp['is_repost'] = True if REPO_TEST_PATTERN.match(
                temp['content']) else False
            try:
                temp['like_num'] = LIKE_NUM_PATTERN.findall(btn_group)[0]
                temp['cmt_num'] = COMMENT_NUM_PATTERN.findall(btn_group)[0]
                temp['repo_num'] = REPO_NUM_PATTERN.findall(btn_group)[0]
            except Exception:
                pass
            cmt = item.select('.cmt')
            # visibility
            if cmt:
                try:
                    temp['visibility'] = VISIBILITY_PATTERN.findall(
                        cmt[0].text)[0]
                except Exception:
                    pass

            # img in main content
            img = item.select('div a img')
            img_src = img[0].attrs['src'] if img else None
            temp['img_src'] = img_src
            LOGGER.debug('img_src: {}'.format(img_src))
            # time & source device
            ct = item.select('span.ct')
            if ct:
                ct = ct[0]
                text = ct.text
                reg_result = TIME_PATTERN.findall(text)[0]

                temp['time'] = ar(
                    '{}年{}'.format(self._current_year, reg_result[0]),
                    DATE_FMTS[0]
                ).naive if reg_result[0] else ar(
                    reg_result[1], DATE_FMTS[1]
                ).naive
                temp['source'] = SOURCE_DEVICE_PATTERN.findall(text)[0]
            self._post_item = Post(**temp)
            self._attachment_item = Attachment(
                uri=img_src, post=self._post_item)
            self._store()

    def _store(self):
        """Commit to SQL database.
        Can be async.
        """
        self._post_item.save()
        self._attachment_item.save()
        self._marshaller.marshall(self._post_item)

    def _dump(self):
        """Dump JSON object to document database.
        """
        with open('weibo_dumps.json', 'w+', encoding='utf-8') as f:
            json.dump(self._marshaller.result, f, ensure_ascii=False)

    def start(self):
        """Start spider. Auto exit when no more post.
        """
        if not self._all_page_num:
            self._set_all_page_num()
            LOGGER.debug('overall pages: {}'.format(self._all_page_num))
        with tqdm(total=self._all_page_num) as progress_bar:
            while self.has_post:
                self._get_html()
                self._parse()
                # 10 post each page.
                progress_bar.update(1)
                progress_bar.set_description(
                    'Page #{}'.format(self._current_page))
                sleep(4)
            self._dump()


def main():
    from harbor.models.base import db
    from harbor.models.post import Post
    from harbor.models.attachment import Attachment
    from harbor.constant import HEADERS
    """Entry point
    """
    db.create_tables([Attachment, Post])
    HEADERS['Cookie'] = ''
    uid = ''
    marshaller = 'ghost'
    s = Spider(HEADERS, uid, marshaller)
    try:
        s.start()
    except Exception as e:
        LOGGER.debug(e)
        LOGGER.debug(s._current_page)
        s._dump()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File: harbor/utils.py
# Author: MingshiCai i@unoiou.com
# Created Date: 2019-08-06 10:28:04
# ----
# Last Modified:
# Modified By:
# ----
# Copyright (c) 2019 MingshiCai i@unoiou.com
import re

from arrow import get
from uuid import uuid1

from harbor.constant import DIRTY_PATTERN, FILTER_PATTERNS


def uuid():
    """Get a uuid str.
    """
    return str(uuid1())


def excerpt(given_str):
    """Generate excerpt.
    TODO:
    """
    return re.split(
        r'[\W]', given_str)[0] + '...' if len(given_str) > 4 else None


def get_milliseconds(time=None):
    """Get milliseconds. Return current timestamp by default.
    Args:
        time: `datetime.datetme`

    Return:
        an `int`
    """
    t = get(time or get())
    return t.timestamp * 1000 + int(t.format('SSS'))


def filter_words(given_str):
    """Filter illegal words.
    TODO:
    """
    return purified_string(given_str, FILTER_PATTERNS)


def https_and_large(given_url):
    """HTTP to HTTPS and large
    TODO: large pic require login/token
    """
    return given_url.replace(
        'http://', 'https://'
    ) if given_url else None


def purified_string(given_str, pattern=DIRTY_PATTERN):
    """Remove newline, spaces
    """
    return re.sub(pattern, '', given_str)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File: WeiboBot\actions.py
# Author: MingshiCai i@unoiou.com
# Date: 2019-12-18 00:59:34
import json
import os
import re
import shutil

from os import getenv
from requests import Session
from selenium import webdriver
from time import sleep

from harbor.marshallers.mobile_marshaller import MobileMarshaller


class Actions:
    """Class for weibo actions.

    Note that you must call `init` and `login` first.
    """
    driver = None  # set default driver to get intelisense

    base_url = 'https://m.weibo.com'
    weibo_api = (
        "https://m.weibo.cn/api/container/getIndex?containerid={}"
        "-_WEIBO_SECOND_PROFILE_WEIBO{}"
    )  # &page_type=03&page={}
    comment_api = (
        "https://m.weibo.cn/comments/hotflow"
        "?id={}&mid={}&max_id_type=0"  # mid from card
    )
    user_info_api = (
        "https://m.weibo.cn/profile/info?uid={}"
    )
    likes_api = (
        "https://m.weibo.cn/api/attitudes/show?id={}&page=1"  # mid
    )
    repost_api = (
        "https://m.weibo.cn/api/statuses/repostTimeline?id={}&page=1"  # mid
    )

    container_id = ''
    uid = ''

    driver_path = getenv('CHROMEDRIVER_PATH', './chromedriver.exe')
    all_weibos_count = 0
    session = Session()

    @classmethod
    def find_css(cls, selector, single=True):
        """Find element by css selector.

        Args:
            selector: (str) css selector
            single: (boolean) whether select single or multiple elements

        Returns:
            (Element or [Element, ])
        """
        if single:
            return cls.driver.find_element_by_css_selector(selector)
        return cls.driver.find_elements_by_css_selector(selector)

    @classmethod
    def find_xpath(cls, xpath, single=True):
        """Find element by xpath.

        Args:
            xpath: (str) xpath
            single: (boolean) whether select single or multiple elements

        Returns:
            (Element or [Element, ])
        """
        if single:
            return cls.driver.find_element_by_xpath(xpath)
        return cls.driver.find_element_by_xpath(xpath)

    @classmethod
    def wait(cls, seconds=1):
        sleep(seconds)
        return cls

    @classmethod
    def init(cls, directory, headless=False):
        """Init weibo bot.

        Args:
            directory: (str) driver default directory
            headless: (boolean) whether run selenium in headless mode
        """
        cls.directory = directory

        chromeOptions = webdriver.ChromeOptions()
        chromeOptions.add_experimental_option(
            "prefs", {
                "download.default_directory": cls.directory,
                "download.prompt_for_download": False,
                "download.directory_upgrade": True, "safebrowsing.enabled": True
            })
        if headless:
            chromeOptions.add_argument("headless")
            chromeOptions.add_argument('window-size=1200x800')
        cls.driver = webdriver.Chrome(
            cls.driver_path, chrome_options=chromeOptions
        )
        return cls

    @classmethod
    def login(cls, loginname, password):
        """Login and set request session cookies.

        Args:
            loginname: (str)
            password: (str)
        """
        cls.driver.get(cls.base_url)
        sleep(2)
        cls.find_xpath("//a[contains(text(),'登录/注册')]").click()
        sleep(2)
        cls.find_xpath("//a[contains(text(),'用帐号密码登录')]").click()
        sleep(2)
        cls.find_xpath("//input[@id='loginName']").clear()
        cls.find_xpath("//input[@id='loginName']").send_keys(loginname)
        cls.find_xpath('//*[@id="loginPassword"]').clear()
        cls.find_xpath('//*[@id="loginPassword"]').send_keys(password)
        cls.find_xpath("//a[@id='loginAction']").click()
        sleep(2)

        return cls

    @classmethod
    def set_user_credentials(cls):
        """Set container id and uid."""
        cls.click_btn_profile().wait()
        cls.uid = re.findall(r'\d+', cls.driver.current_url)[0]
        cls.scroll_to_bottom().click_btn_view_all().wait(2)
        cls.container_id = re.findall(r'\d+_', cls.driver.current_url)[0]
        cls.weibo_api = cls.weibo_api.format(cls.container_id, '{}')
        return cls

    @classmethod
    def set_cookies(cls):
        """Set session cookies with webdriver's current cookies."""
        cls.session.cookies.clear()
        for cookie in cls.driver.get_cookies():
            cls.session.cookies.set(cookie['name'], cookie['value'])
        return cls

    @classmethod
    def patch_post_extra_info(cls, post_item):
        """Patch reposts, likes and comments data to post item.

        Args:
            post_item: (dict)

        Returns:
            (dict or None)
        """
        if not post_item:
            return None
        mid = post_item['id']
        for key, api, extra_mid in [
            ['comments', cls.comment_api, True],
            ['likes', cls.likes_api, False],
            ['reposts', cls.repost_api, False]
        ]:
            url = api.format(*(mid, mid) if extra_mid else mid)
            post_item[key] = cls.session.get(url).json()
        return post_item

    @classmethod
    def download_original_image(cls, urls, mid):
        """Download and save image by weibo id.

        Args:
            urls: ([str, ]) image urls list
            mid: (str) weibo mid
        """
        pic_dir = './download/users/{}/images/{}'.format(cls.uid, mid)
        if not os.path.exists(pic_dir):
            os.makedirs(pic_dir)
        for pic_index, url in enumerate(urls):
            res = cls.session.get(url, stream=True)
            pic_suffix = url.split('.')[-1]
            with open(
                '{}/{}.{}'.format(pic_dir, pic_index, pic_suffix), 'wb'
            ) as pic_file:
                res.raw.decode_content = True
                shutil.copyfileobj(res.raw, pic_file)

    @classmethod
    def get_posts_single_page(cls, page_index=None):
        """Get posts in single page.

        Args:
            page_index: (int) page index

        Returns:
            [dict, ]
        """
        suffix = '&page_type=03&page={}'.format(
            page_index) if page_index else ''
        res = cls.session.get(cls.weibo_api.format(suffix))
        cards = res.json()['data']['cards']
        posts = []
        for card in cards:
            post = cls.patch_post_extra_info(MobileMarshaller.mobile_card(card))
            if post:
                if post['pic_num'] > 0:
                    pic_urls = [item['large']['url'] for item in post['pics']]
                    cls.download_original_image(pic_urls, post['id'])
                posts.append(post)
        cls.cache_posts(posts, page_index)
        cls.wait(10)
        return posts

    @classmethod
    def cache_posts(cls, posts, page_index):
        """Save posts locally.

        Args:
            posts: ([dict, ]) list of posts
            page_index: (int)
        """
        with open(
            './download/users/{}/posts/{}.json'.format(cls.uid, page_index),
            'w+', encoding='UTF8'
        ) as json_file:
            json.dump(posts, json_file, ensure_ascii=False)

    @classmethod
    def click_btn_profile(cls):
        """Click the nav btn to go to homepage.
        """
        cls.find_css('.nav-left').click()
        return cls

    @classmethod
    def all_weibos_count(cls):
        """Get current user's all weibos count"""
        return int(cls.session.get(
            cls.user_info_api.format(cls.uid)
        ).json()['data']['user']['statuses_count'])

    @classmethod
    def click_btn_view_all(cls):
        """Go to all weibos page.
        """
        cls.find_css('.lite-btn-more').click()
        return cls

    @classmethod
    def scroll_to_bottom(cls):
        cls.driver.execute_script(
            "window.scrollTo(0, document.body.scrollHeight)"
        )
        return cls



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File: harbor/constant.py
# Author: MingshiCai i@unoiou.com
# Created Date: 2019-08-06 10:28:15
# ----
# Last Modified:
# Modified By:
# ----
# Copyright (c) 2019 MingshiCai i@unoiou.com
import re

FILTER_PATTERNS = [
    re.compile(r'反动')
]
DIRTY_PATTERN = re.compile(r'(\\n)|(\s+)')

URL_TPL = 'https://weibo.cn/{}/profile?page={}'
HEADER_PATH_TPL = '/{}/profile?page={}'
REFERER_URL_TPL = 'https://weibo.cn/{}/profile?page={}'

LIKE_NUM_PATTERN = re.compile(r'赞\[(\d+)\]')
COMMENT_NUM_PATTERN = re.compile(r'评论\[(\d+)\]')
REPO_NUM_PATTERN = re.compile(r'转发\[(\d+)\]')
REPO_TEST_PATTERN = re.compile(r'(转发了)|(Repost)')
TIME_PATTERN = re.compile(
    r'(\d+月\d+日 [\d\:]+)|(\d{4}\-\d{2}\-\d{2} [\d\:]+)')
SOURCE_DEVICE_PATTERN = re.compile(r'(来自.+)')
VISIBILITY_PATTERN = re.compile(r'\[([仅自].+)\]')

DATE_FMTS = (
    'YYYY年MM月DD日 HH:mm', 'YYYY-MM-DD HH:mm:ss'
)

HEADERS = {
    'authority': 'weibo.cn',
    'method': 'GET',
    'scheme': 'https',
    'dnt': '1',
    'upgrade-insecure-requests': '1',
    'user-agent': ('Mozilla/5.0 (iPhone; CPU iPhone OS 11_0 like Mac OS X) '
                   'AppleWebKit/604.1.38 (KHTML, like Gecko) Version/11.0 '
                   'Mobile/15A372 Safari/604.1'),
    'Cookie': None  # set to your own cookie
}



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File: marshaller\mobile_marshaller.py
# Author: MingshiCai i@unoiou.com
# Date: 2019-12-18 10:40:47


class MobileMarshaller:
    """Class for marshalling weibo json object.
    """
    mobile_card_types = {
        9: 'post', 31: 'search-box'
    }
    mobile_card_keys = [
        'attitudes_count', 'pics', 'pic_num', 'id', 'created_at',
        'comments_count', 'favorited', 'reads_count', 'reposts_count',
        'source', 'text', 'textLength', 'visible', 'weibo_position',
        ['title', 'text']
    ]
    @classmethod
    def mobile_card(cls, card_object):
        """Extract main content from weibo h5 json card.

        Args:
            card_object: (dict) card item from weibo api

        Returns:
            ({str: object} or None)
        """
        if cls.mobile_card_types.get(card_object['card_type'], None) != 'post':
            return None
        result = {}
        card = card_object['mblog']
        for key in cls.mobile_card_keys:
            if isinstance(key, str):
                try:
                    result[key] = card[key]
                except Exception as e:
                    print(e, str(e))
                    pass
            else:
                tmp = card
                for sub_key in key:
                    tmp = tmp[sub_key]
                result['.'.join(key)] = tmp

        return result



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File: marshaller/__init__.py
# Author: MingshiCai i@unoiou.com
# Created Date: 2019-08-06 10:35:27
# ----
# Last Modified:
# Modified By:
# ----
# Copyright (c) 2019 MingshiCai i@unoiou.com



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File: harbor/main.py
# Author: MingshiCai i@unoiou.com
# Created Date: 2019-08-06 10:27:19
# ----
# Last Modified:
# Modified By:
# ----
# Copyright (c) 2019 MingshiCai i@unoiou.com
import logging

from os import getenv

from harbor.spider.mobile_bot import WeiboBot

LOGGER = logging(__name__)


def main():
    save_to_directory = './weibo_backup'
    LOGGER.info('start mobile weibo bot')
    bot = WeiboBot(
        save_to_directory,
        getenv('WEIBO_USERNAME', 'default-username'),
        getenv('WEIBO_PASSWORD', 'default-password'),
    )
    bot.save_posts()
    LOGGER.info('exit normally')


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File: WeiboBot\wbbot.py
# Last Edited By: MingshiCai i@unoiou.com
# Date: 2019-12-15 00:08:57
from os import getenv
from tqdm import tqdm

from harbor.actions import Actions


class WeiboBot:
    """Class for weibo bot.
    """

    def __init__(self, directory, loginname, password, headless=False):
        """Init weibo bot.

        Args:
            directory
            headless
        """
        self.acts = Actions.init(directory, headless)
        self.acts.login(loginname, password)

    def save_posts(self):
        """Get all weibo content."""
        self.acts.wait().set_user_credentials().set_cookies()
        all_weibos_count = self.acts.all_weibos_count()
        progress = tqdm(total=all_weibos_count, desc='all weibos')
        posts = []
        page_index = 1
        retry = 3
        while len(posts) < all_weibos_count and retry > 0:
            try:
                posts.extend(self.acts.get_posts_single_page(page_index))
            except Exception as e:
                print(e)
                retry -= 1
            else:
                page_index += 1
                progress.update(len(posts))
                retry = 3


def main():
    save_to_directory = './weibo_backup'

    bot = WeiboBot(
        save_to_directory,
        getenv('WEIBO_USERNAME', 'default-username'),
        getenv('WEIBO_PASSWORD', 'default-password'),
    )
    bot.save_posts()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File: harbor/setup.py
# Author: MingshiCai i@unoiou.com
# Created Date: 2019-08-06 10:05:34
# ----
# Last Modified:
# Modified By:
# ----
# Copyright (c) 2019 MingshiCai i@unoiou.com
"""Set up modules
"""
from setuptools import setup, find_packages

setup(
    name='weibo-harbor',
    version='0.1.0',
    description='A package to crawl, backup and export weibo content.',
    author='MingshiCai',
    author_email='i@unoiou.com',
    url='https://github.com/cmsax/harbor',

    packages=find_packages(),

    install_requires=[
        'arrow',
        'peewee',
        'requests',
        'selenium',
        'BeautifulSoup4',
        'tqdm',
        'nose',
        'flake8'
    ],

    entry_points={
        'console_scripts': ['harbor = harbor.main:main']
    }
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File: marshaller/factory.py
# Author: MingshiCai i@unoiou.com
# Created Date: 2019-08-06 10:34:37
# ----
# Last Modified:
# Modified By:
# ----
# Copyright (c) 2019 MingshiCai i@unoiou.com
from typing import NewType

from harbor.utils import (
    uuid, get_milliseconds, excerpt, https_and_large, purified_string
)
from harbor.models.post import Post

PostItem = NewType('Post', Post)  # this doesn't work yet.


class Marshaller:
    """Post marshaller base class.
    """

    available_marshallers = ['ghost']
    ghost_version = '2.14.0'

    def __init__(self, marshaller):
        """Init

        Args:
            marshaller: `str` marshaller name
        """
        if marshaller not in self.available_marshallers:
            raise KeyError('Given marshaller key is not available.')
        self._marshaller = marshaller
        self._post_id = 100  # to avoid duplicated entry, set with a big num.
        self._results = []
        self._posts = []
        self._json = {}

    @property
    def result(self):
        """Marshalled dict obj.

        Return:
            a dict
        """
        getattr(self, '_{}_json'.format(self._marshaller))()
        return self._json

    @property
    def posts(self):
        """
        Return:
            a list of dict
        """
        return self._posts

    def marshall(self, post_obj: PostItem):
        """Marshall post object.

        Args:
            post_obj: `models.Post`
        """
        getattr(self, '_{}'.format(self._marshaller))(post_obj)

    def _ghost_json(self):
        """JSON marshaller for ghost.
        """
        tag_id = 2
        user_id = 99
        weibo_user_name = 'Latina_XXX'  # customize

        self._json = {
            'meta': {
                'exported_on': get_milliseconds(),
                'version': self.ghost_version
            },
            'data': {
                'posts': self._posts,
                'tags': [{
                    'id': tag_id,
                    'name': '微博',
                    'slug': 'weibo',
                    'description': ''  # customize it if necessary
                }],
                'posts_tags': [
                    {'tag_id': tag_id, 'post_id': post_item['id']}
                    for post_item in self._posts
                ],
                'users': [{
                    "id": user_id,
                    "name": weibo_user_name,
                    "slug": '_'.join(weibo_user_name.split(' ')).lower(),
                    "email": "i@unoiou.com",
                    "profile_image": None,
                    "cover_image": None,
                    "bio": None,
                    "website": None,
                    "location": None,
                    "accessibility": None,
                    "meta_title": None,
                    "meta_description": None,
                    "created_at": get_milliseconds(),
                    "created_by": user_id,
                    "updated_at": get_milliseconds(),
                    "updated_by": user_id
                }],
            }
        }

    def _ghost(self, post_obj: PostItem):
        """Marshaller for Ghost blog engine.

        Args:
            post_obj: `models.Post`

        tags, post_tags, posts, users :: content, date, img_src, visibility
        """
        temp = {
            'id': self._post_id,
            'title': excerpt(post_obj.content) or '无题',
            'slug': '/weibo-{}'.format(uuid()),
            'mobiledoc': (
                "{\"version\":\"0.3.1\",\"atoms\":[],\"cards\":[],"
                "\"markups\":[],\"sections\":[[1,\"p\","
                "[[0,[],0,\"" + purified_string(post_obj.content) + "\"]]]]}"
            ),
            'plaintext': post_obj.content,
            'feature_image': https_and_large(post_obj.img_src),
            "page": 0,
            "featured": 0,
            "status": "published",
            "published_at": get_milliseconds(post_obj.time),
            "published_by": 1,
            "meta_title": None,
            "meta_description": None,
            "author_id": 1,
            "created_at": get_milliseconds(post_obj.time),
            "created_by": 1,
            "updated_at": get_milliseconds(post_obj.time),
            "updated_by": 1
        }
        self._posts.append(temp)
        self._post_id += 1



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File: models/post.py
# Author: MingshiCai i@unoiou.com
# Created Date: 2019-08-06 10:33:20
# ----
# Last Modified:
# Modified By:
# ----
# Copyright (c) 2019 MingshiCai i@unoiou.com
from peewee import (CharField, IntegerField, TextField, DateTimeField,
                    BooleanField)

from harbor.models.base import BaseModel


class Post(BaseModel):
    content = TextField()
    time = DateTimeField()
    like_num = IntegerField(null=True)
    comment_num = IntegerField(null=True)
    repost_num = IntegerField(null=True)
    source = CharField(max_length=20)
    visibility = CharField(max_length=10, null=True)
    img_src = CharField(max_length=100, null=True)
    is_repost = BooleanField(null=True)
